/*-
 * Copyright (c) 2005 Daniel Braniss <danny@cs.huji.ac.il>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
/*
 | iSCSI
 | $Id: isc_subr.c,v 1.17 2006/02/27 07:08:40 danny Exp danny $
 */
#include <sys/param.h>
#include <sys/kernel.h>
#include <sys/conf.h>
#include <sys/systm.h>
#include <sys/malloc.h>
#include <sys/ctype.h>
#include <sys/errno.h>
#include <sys/sysctl.h>
#include <sys/file.h>
#include <sys/uio.h>
#include <sys/socketvar.h>
#include <sys/socket.h>
#include <sys/protosw.h>
#include <sys/proc.h>
#include <sys/ioccom.h>
#include <sys/queue.h>
#include <sys/kthread.h>
#include <sys/syslog.h>
#include <sys/mbuf.h>

#include <dev/iscsi/iscsi.h>
#include <dev/iscsi/iscsivar.h>

MALLOC_DEFINE(M_PDU, "iSCSI pdu", "iSCSI driver");

void
i_flush(isc_session_t *sp)
{
     int	n = 0;
     pduq_t	*pq;

     while((pq = i_dqueue_rsp(sp)) != NULL) {
	  pdu_free(sp->isc, pq);
	  n++;
     }
     while((pq = i_dqueue_rsv(sp)) != NULL) {
	  pdu_free(sp->isc, pq);
	  n++;
     }
     while((pq = i_dqueue_snd(sp, -1)) != NULL) {
	  pdu_free(sp->isc, pq);
	  n++;
     }
     while((pq = i_dqueue_hld(sp)) != NULL) {
	  pdu_free(sp->isc, pq);
	  n++;
     }
     sdebug(5, "%d pdus recovered", n);
}

static char *
i_strdupin(char *s, size_t maxlen)
{
     size_t	len;
     char	*p, *q;

     p = malloc(maxlen, M_ISCSI, M_WAITOK);
     copyinstr(s, p, maxlen, &len);
     q = malloc(len, M_ISCSI, M_WAITOK);
     bcopy(p, q, len);
     free(p, M_ISCSI);

     return q;
}
/*
 | XXX: not finished coding
 */
int
i_setopt(isc_session_t *sp, isc_opt_t *opt)
{
     if(opt->maxRecvDataSegmentLength > 0) {
	  sp->opt.maxRecvDataSegmentLength = opt->maxRecvDataSegmentLength;
	  sdebug(2, "maxRecvDataSegmentLength=%d", sp->opt.maxRecvDataSegmentLength);
     }
     if(opt->maxBurstLength != 0) {
	  sp->opt.maxBurstLength = opt->maxBurstLength;
	  sdebug(2, "maxBurstLength=%d", sp->opt.maxBurstLength);
     }

     if(opt->targetAddress != NULL) {
	  if(sp->opt.targetAddress != NULL)
	       free(sp->opt.targetAddress, M_ISCSI);
	  sp->opt.targetAddress = i_strdupin(opt->targetAddress, 128);
	  sdebug(4, "opt.targetAddress='%s'", sp->opt.targetAddress);
     }
     if(opt->targetName != NULL) {
	  if(sp->opt.targetName != NULL)
	       free(sp->opt.targetName, M_ISCSI);
	  sp->opt.targetName = i_strdupin(opt->targetName, 128);
	  sdebug(4, "opt.targetName='%s'", sp->opt.targetName);
     }
     if(opt->initiatorName != NULL) {
	  if(sp->opt.initiatorName != NULL)
	       free(sp->opt.initiatorName, M_ISCSI);
	  sp->opt.initiatorName = i_strdupin(opt->initiatorName, 128);
	  sdebug(4, "opt.initiatorName='%s'", sp->opt.initiatorName);
     }

     if(opt->maxLuns > 0) {
	  if(opt->maxLuns > ISCSI_MAX_LUNS)
	       sp->opt.maxLuns = ISCSI_MAX_LUNS; // silently chop it down ...
	  sp->opt.maxLuns = opt->maxLuns;
	  sdebug(4, "opt.maxLuns=%d", sp->opt.maxLuns);
     }

     return 0;
}

void
i_freeopt(isc_opt_t *opt)
{
     if(opt->targetAddress != NULL) {
	  free(opt->targetAddress, M_ISCSI);
	  opt->targetAddress = NULL;
     }
     if(opt->targetName != NULL) {
	  free(opt->targetName, M_ISCSI);
	  opt->targetName = NULL;
     }
     if(opt->initiatorName != NULL) {
	  free(opt->initiatorName, M_ISCSI);
	  opt->initiatorName = NULL;
     }
}
